@@ -1,5 +1,12 @@
 
-/* 	$Id: rconifers.c 674 2010-10-25 22:14:00Z hamannj $	 */
+/* 	$Id: rconifers.c 862 2012-02-22 21:29:18Z hamannj $	 */
+
+/* to build the R package:	*/
+/* open mingwn shell		*/
+/* cd c:\conifers			*/
+/* r cmd check rconfiers	*/
+/* r cmd build rconifers	*/
+/* r cmd binary rconifers	*/
 
 /* To submit to CRAN: */
 /* $ ch to conifers directory */
@@ -32,13 +39,21 @@
 /* 221 Goodbye. */
 /* $  */
 
+// $ svn checkout http://www.forestinformatics.com/conifers
+// $ cd /conifers/trunk
+// Win32:
+// $ RCMD build --no-manual --binary rconifers
+// other
+// $ R CMD build --no-manual --binary rconifers
+
 
 
 /* and send email to cran@r-project.org.  */
 /* Please indicate the copyright situation (GPL, ...) in your submission.  */
 
 
-
+/* don't forget to run doxygen on the source code as well to generate the software docs */
+/* http://www.digilife.be/quickreferences/QRC/Doxygen%20Quick%20Reference.pdf */
 
 #include <math.h>
 #include <memory.h>
@@ -62,6 +77,8 @@
 unsigned long current_variant;
 //unsigned long use_genetic_gains;
 
+double MODEL_VERSION;
+
 double COEFFS_VERSION;
 unsigned long N_COEFFS;
 struct COEFFS_RECORD *COEFFS_PTR;
@@ -80,27 +97,30 @@ SEXP getvar(SEXP name, SEXP rho);
 /* new functions for next version of the library */
 SEXP r_init_variant( SEXP variant_sexp );
 SEXP r_set_variant( SEXP variant_sexp );
-/* SEXP r_get_mort_settings(); */
-/* SEXP r_set_mort_settings(  SEXP map_sexp, SEXP verbose_sexp ); */
 SEXP r_set_species_map(  SEXP map_sexp, SEXP verbose_sexp  );
 SEXP r_reseed( SEXP ctl );
 SEXP r_project_sample( SEXP data_sexp, SEXP n_years_sexp, SEXP ctl_sexp );
 SEXP r_thin_sample( SEXP data_sexp,   SEXP ctl_sexp );
-SEXP r_fill_in_missing_values( SEXP data_sexp, SEXP ctl_sexp ) ;
+SEXP r_impute_missing_values( SEXP data_sexp, SEXP ctl_sexp ) ;
 SEXP r_calc_max_sdi( SEXP data_sexp );
 
 /* these functions are used to convert the plots between the two interfaces */
 struct PLOT_RECORD *build_plot_array_from_sexp( SEXP plot_sexp, 
 						unsigned long *n_plots );
+
 SEXP build_sexp_from_plot_array( unsigned long n_plots, 
 				 struct PLOT_RECORD *plots_ptr );
 
 struct PLANT_RECORD *build_plant_array_from_sexp( SEXP plant_sexp, 
 						  unsigned long *n_plants );
+
 SEXP build_sexp_from_plant_array( unsigned long n_plants, 
 				  struct PLANT_RECORD *plants_ptr );
+
 SEXP build_return_data_sexp( double x0,
 			     unsigned long age,
+			     unsigned long n_years_projected,
+
 			     unsigned long n_plots,
 			     struct PLOT_RECORD *plots_ptr,
 			     unsigned long n_plants,
@@ -111,6 +131,9 @@ SEXP build_return_data_sexp( double x0,
 SEXP init_conifers();
 SEXP exit_conifers();
 
+/* a function to print the variant label */
+char *variant_label( unsigned long variant );
+
 
 /************************************************************************/
 /* function definitions							*/
@@ -175,7 +198,12 @@ SEXP r_set_variant( SEXP variant_sexp )
 
    PROTECT(ans = allocVector(INTSXP, 1));
 
-   if( variant == CONIFERS_SWO || variant == CONIFERS_SMC )
+   if( variant == CONIFERS_SWO || 
+	   variant == CONIFERS_SMC ||
+	   variant == CONIFERS_SWOHYBRID )
+//	   variant == CONIFERS_SWOHYBRID ||
+//	   variant == CONIFERS_CIPS
+//	   )
    {
       /* the coefficients section */
       if( COEFFS_PTR )
@@ -186,12 +214,13 @@ SEXP r_set_variant( SEXP variant_sexp )
       }   
       
       /* load the coeffs for the variant */
-      COEFFS_PTR  = con_init_coeffs( variant, &N_COEFFS, &COEFFS_VERSION );
+      COEFFS_PTR  = con_init_coeffs( variant, &N_COEFFS, &COEFFS_VERSION, &MODEL_VERSION );
    }
    else
    {
+     // Rprintf( "The only variants allowed are: zero (0=CONIFERS_SWO), one (1=CONIFERS_SMC), two (2=CONIFERS_SWOHYBRID), and three (3=CONIFERS_CIPS)\n" );
+        Rprintf( "The only variants allowed are: zero (0=CONIFERS_SWO), one (1=CONIFERS_SMC), and two (2=CONIFERS_SWOHYBRID)\n" );
       error( "That variant is invalid\n" );
-      error( "Only variant zero (0) or one (1) is valid\n" );
       INTEGER(ans)[0] = 1;
       UNPROTECT(1);
       return ans;
@@ -208,8 +237,32 @@ SEXP r_set_variant( SEXP variant_sexp )
 
    if( COEFFS_PTR )
    {
-      Rprintf( "Initialized %ld functional species coefficients for variant # %ld\n", N_COEFFS, variant );
-      Rprintf( "coefficients version is %lf\n", COEFFS_VERSION );
+      Rprintf( "Initialized %ld functional species coefficients for variant # %ld %s\n", 
+	       N_COEFFS, variant, variant_label(variant) );
+      //Rprintf( "Initialized %ld functional species coefficients for variant # %ld %s\n", N_COEFFS, variant );
+      
+      switch (variant)
+      {
+	 case CONIFERS_SWO:
+	    Rprintf( "The code label for the variant is CONIFERS_SWO\n" );
+	    break;
+	 case CONIFERS_SMC:
+	    Rprintf( "The code label for the variant is CONIFERS_SMC\n" );
+	    break;
+	 case CONIFERS_SWOHYBRID:
+	    Rprintf( "The code label for the variant is CONIFERS_SWOHYBRID\n" );
+	    break;
+//	 case CONIFERS_CIPS:
+	 //   Rprintf( "The code label for the variant is CONIFERS_CIPS\n" );
+//	    break;
+	 default:
+	    Rprintf( "There is no code label for the variant you supplied\n" );
+	    break;
+      }
+      
+      Rprintf( "The coefficients version is %lf\n", COEFFS_VERSION );
+      Rprintf( "The model version is %lf\n", MODEL_VERSION );
+
    }
 
    /* set the global variable for the variant */
@@ -226,6 +279,33 @@ SEXP r_set_variant( SEXP variant_sexp )
 }
 
 
+char *variant_label( unsigned long variant )
+{
+
+      switch (variant)
+      {
+	 case CONIFERS_SWO:
+	    return "CONIFERS_SWO";
+	    break;
+	 case CONIFERS_SMC:
+	    return "CONIFERS_SMC";
+	    break;
+	 case CONIFERS_SWOHYBRID:
+	    return "CONIFERS_SWOHYBRID";
+	    break;
+//	 case CONIFERS_CIPS:
+//	    return "CONIFERS_CIPS";
+//	    break;
+	 default:
+	    return "CONIFERS_SWO";
+	    break;
+      }
+
+}       
+
+
+
+
 /* i think you should convert this read from a data.frame object */
 SEXP  r_set_species_map(  SEXP map_sexp, 
 			  SEXP verbose_sexp )
@@ -246,6 +326,10 @@ SEXP  r_set_species_map(  SEXP map_sexp,
    SEXP genetic_worth_h_sexp;
    SEXP genetic_worth_d_sexp;
 
+   SEXP min_temp_sexp;
+   SEXP max_temp_sexp;
+   SEXP opt_temp_sexp;
+
    /* protect in incomming list */
    PROTECT( map_sexp = AS_LIST( map_sexp ) );
    PROTECT(ans = allocVector(INTSXP, 1));
@@ -265,6 +349,11 @@ SEXP  r_set_species_map(  SEXP map_sexp,
    genetic_worth_h_sexp = get_list_element( map_sexp, "gwh" );
    genetic_worth_d_sexp = get_list_element( map_sexp, "gwd" );
 
+   /* added species specific temperature variables */
+   min_temp_sexp = get_list_element( map_sexp, "mint" );
+   max_temp_sexp = get_list_element( map_sexp, "maxt" );
+   opt_temp_sexp = get_list_element( map_sexp, "optt" );
+
    /* coerce the vectors */
    PROTECT( idx_sexp = coerceVector( idx_sexp, INTSXP ) );
    PROTECT( fsp_sexp = coerceVector( fsp_sexp, INTSXP ) );
@@ -276,6 +365,12 @@ SEXP  r_set_species_map(  SEXP map_sexp,
    PROTECT( genetic_worth_h_sexp = coerceVector( genetic_worth_h_sexp, REALSXP ) );
    PROTECT( genetic_worth_d_sexp = coerceVector( genetic_worth_d_sexp, REALSXP ) );
 
+   /* added species specific temperature variables */
+   PROTECT( min_temp_sexp = coerceVector( min_temp_sexp, REALSXP ) );
+   PROTECT( max_temp_sexp = coerceVector( max_temp_sexp, REALSXP ) );
+   PROTECT( opt_temp_sexp = coerceVector( opt_temp_sexp, REALSXP ) );
+
+
    if( SPECIES_PTR )
    {
       free( SPECIES_PTR );
@@ -289,7 +384,7 @@ SEXP  r_set_species_map(  SEXP map_sexp,
    {
       N_SPECIES = 0;
       INTEGER(ans)[0] = 0;
-      UNPROTECT(12);
+      UNPROTECT(15);
       return ans;
    }
 
@@ -307,13 +402,17 @@ SEXP  r_set_species_map(  SEXP map_sexp,
       SPECIES_PTR[i].genetic_worth_h = REAL( genetic_worth_h_sexp )[i];
       SPECIES_PTR[i].genetic_worth_d = REAL( genetic_worth_d_sexp )[i];
 
+      SPECIES_PTR[i].min_temp = REAL( min_temp_sexp )[i];
+      SPECIES_PTR[i].max_temp = REAL( max_temp_sexp )[i];
+      SPECIES_PTR[i].opt_temp = REAL( opt_temp_sexp )[i];
+
       if( verbose )
       {
-	 Rprintf( "%i,%i,%s,%f\n",
-	 SPECIES_PTR[i].idx,
-	 SPECIES_PTR[i].fsp_idx,
-	 SPECIES_PTR[i].sp_code,
-	 SPECIES_PTR[i].max_sdi
+		 Rprintf( "%i,%i,%s,%f\n",
+			 SPECIES_PTR[i].idx,
+			 SPECIES_PTR[i].fsp_idx,
+			 SPECIES_PTR[i].sp_code,
+			 SPECIES_PTR[i].max_sdi
 	    );
       }
 
@@ -326,7 +425,11 @@ SEXP  r_set_species_map(  SEXP map_sexp,
 	   compare_species_by_idx );
 
    INTEGER(ans)[0] = N_SPECIES;
-   UNPROTECT(12);
+
+   //UNPROTECT(12);
+   /* this argument must equal the same number of PROTECT calls for the function */
+   UNPROTECT(15);
+
    return ans;
 
 }
@@ -394,9 +497,9 @@ SEXP getvar(SEXP name, SEXP rho)
 
 
 SEXP r_project_sample( 
-   SEXP data_sexp,
-   SEXP n_years_sexp,
-   SEXP ctl_sexp ) 
+   SEXP data_sexp,      /* stand, tree, and plot level variables                */
+   SEXP n_years_sexp,   /* number of years to project the plant list            */
+   SEXP ctl_sexp )      /* control list of variables for controlling simulator  */ 
 {
 
    unsigned long return_code;
@@ -419,6 +522,8 @@ SEXP r_project_sample(
    unsigned long hcb_growth_on = 1;
    double x0;
    unsigned long age = 0;
+   unsigned long n_years_projected = 0;
+
 
    SEXP ret_val;
 
@@ -428,24 +533,24 @@ SEXP r_project_sample(
 	   sizeof( struct SPECIES_RECORD ),
 	   compare_species_by_idx );
 
+   /* get the stand level variables */
+   x0 = asReal( get_list_element( data_sexp, "x0" ) );
+   age = asInteger( get_list_element( data_sexp, "age" ) );
+   n_years_projected = asInteger( get_list_element( data_sexp, "n.years.projected" ) );
 
-   /* intitialize the config variables, grab them from the ctl list */
+   if( x0 < 0.0 )
+   {
+      x0 = 0.0;
+   }
+
+
+   /* intitialize the config/control variables (ctl argument) */
    rand_error  = asInteger( get_list_element( ctl_sexp, "rand.err" ) );
    rand_seed  = asInteger( get_list_element( ctl_sexp, "rand.seed" ) );
    endemic_mort = asInteger( get_list_element( ctl_sexp, "endemic.mort" ) );
    sdi_mort  = asInteger( get_list_element( ctl_sexp, "sdi.mort" ) ); 
    use_genetic_gains  = asInteger( get_list_element( ctl_sexp, "genetic.gains" ) );
 
-   
-   /* all bet this damn variable is causing the troubles!!!! */
-   /* this is now going into the data list object */
-   x0 = asReal( get_list_element( data_sexp, "x0" ) );
-   age = asInteger( get_list_element( data_sexp, "age" ) );
-
-   if( x0 < 0.0 )
-   {
-      x0 = 0.0;
-   }
 
 /*    Rprintf( "value of x0 = %lf\n", x0 ); */
 /*    Rprintf( "value of age = %ld\n", age ); */
@@ -473,6 +578,7 @@ SEXP r_project_sample(
 	       Rprintf( "Invalid plots for variant %ld. Check site index values\n", current_variant );
 	       ret_val = build_return_data_sexp( x0,
 						 age,
+                         n_years_projected,
 						 n_plots, plots_ptr, 
 						 n_plants, plants_ptr  );  
 	       
@@ -524,7 +630,31 @@ SEXP r_project_sample(
 			    use_precip_in_hg,
 			    rand_error,
 			    current_variant,
-			    use_genetic_gains);
+			    use_genetic_gains,
+				age,
+                &n_years_projected );
+
+    /*
+void __stdcall project_plant_list( 
+   unsigned long           *return_code,
+   unsigned long           n_species,
+   struct SPECIES_RECORD   *species_ptr,
+   unsigned long           n_coeffs,
+   struct COEFFS_RECORD    *coeffs_ptr,
+   unsigned long           n_plants,
+   struct PLANT_RECORD     *plants_ptr,
+   unsigned long           n_points,
+   struct PLOT_RECORD      *plots_ptr,
+   double		           *x0, 		   
+   unsigned long           endemic_mortality,      
+   unsigned long           sdi_mortality,          
+   int                     hcb_growth_on,          
+   unsigned long           use_precip_in_hg,       
+   unsigned long           use_rand_err,
+   unsigned long	        variant,
+   unsigned long            use_genetic_gains,
+   unsigned long			plantation_age )
+*/
 
 /* 	Rprintf( "age = %d, value of x0 = %lf, after\n", age, x0 ); */
 
@@ -534,6 +664,7 @@ SEXP r_project_sample(
 	   Rprintf( "unable to project, return_code = %ld, %lf, check conifers.h for list of return codes\n", return_code, x0 );
 	   ret_val = build_return_data_sexp( x0,
 					     age,
+                         n_years_projected,
 					     n_plots, plots_ptr, 
 					     n_plants, plants_ptr  );  
 	  
@@ -555,6 +686,7 @@ SEXP r_project_sample(
 /*   Rprintf( "building return data sexp..." ); */
   ret_val = build_return_data_sexp( x0,
 				    age,
+                    n_years_projected,
 				    n_plots, plots_ptr, 
 				    n_plants, plants_ptr  );  
 /*   Rprintf( "done\n" ); */
@@ -573,6 +705,7 @@ SEXP r_project_sample(
 /* this function seems to be working as intended */
 SEXP build_return_data_sexp( double x0,
 			     unsigned long age,
+			     unsigned long n_years_projected,
 			     unsigned long n_plots,
 			     struct PLOT_RECORD *plots_ptr,
 			     unsigned long n_plants,
@@ -584,7 +717,8 @@ SEXP build_return_data_sexp( double x0,
    SEXP sexp_plots;
    SEXP sexp_plants;
 
-   PROTECT( ret_val = allocVector( VECSXP, 4 ) );
+   /* this protects one more than the vector itself */
+   PROTECT( ret_val = allocVector( VECSXP, 5 ) );
 
 /*    Rprintf( "value of x0 = %lf\n", x0 ); */
    SET_VECTOR_ELT( ret_val, 0, ScalarReal( x0 ) );
@@ -596,11 +730,14 @@ SEXP build_return_data_sexp( double x0,
    PROTECT( sexp_plants =  build_sexp_from_plant_array( n_plants, plants_ptr ) );
    SET_VECTOR_ELT( ret_val, 3, sexp_plants );
 
+   SET_VECTOR_ELT( ret_val, 4, ScalarInteger( n_years_projected ) );
+
    UNPROTECT( 4 );
    return ret_val;
    
 }
 
+/* todo: update the plot array from the new data.frame */
 struct PLOT_RECORD *build_plot_array_from_sexp( SEXP plot_sexp, 
 						unsigned long *n_plots )
 {
@@ -610,31 +747,142 @@ struct PLOT_RECORD *build_plot_array_from_sexp( SEXP plot_sexp,
    
    /* plots s expression variables */
    SEXP plot_plot_sexp;
+
+   SEXP plot_lat_sexp;
+   SEXP plot_lon_sexp;
+
    SEXP plot_elev_sexp;
    SEXP plot_slp_sexp; 
    SEXP plot_asp_sexp; 
    SEXP plot_h20_sexp; 
    SEXP plot_map_sexp;
    SEXP plot_si30_sexp; //site index
+   
+   /* these are the variables for the new SWOHYBRID variants */
+   SEXP plot_gsp_sexp; // growing season precipitation
+
+   /* monthly temps and solar radiation */
+   SEXP plot_mt1_sexp;	/* monthly temp for january */
+   SEXP plot_mt2_sexp;	/* monthly temp for feb */
+   SEXP plot_mt3_sexp;	/* monthly temp for march */
+   SEXP plot_mt4_sexp;	/* monthly temp for april */
+   SEXP plot_mt5_sexp;	/* monthly temp for may  */
+   SEXP plot_mt6_sexp;	/* monthly temp for june */
+   SEXP plot_mt7_sexp;	/* monthly temp for july */
+   SEXP plot_mt8_sexp;	/* monthly temp for aug */
+   SEXP plot_mt9_sexp;	/* monthly temp for sept */
+   SEXP plot_mt10_sexp;	/* monthly temp for oct */
+   SEXP plot_mt11_sexp;	/* monthly temp for nov */
+   SEXP plot_mt12_sexp;	/* monthly temp for dec */
+   
+   SEXP plot_srad1_sexp;	/* solar radiation for january */
+   SEXP plot_srad2_sexp;	/* solar radiation for feb */
+   SEXP plot_srad3_sexp;	/* solar radiation for mar */
+   SEXP plot_srad4_sexp;	/* solar radiation for april */
+   SEXP plot_srad5_sexp;	/* solar radiation for may */
+   SEXP plot_srad6_sexp;	/* solar radiation for june */
+   SEXP plot_srad7_sexp;	/* solar radiation for july */
+   SEXP plot_srad8_sexp;	/* solar radiation for aug */
+   SEXP plot_srad9_sexp;	/* solar radiation for sept */
+   SEXP plot_srad10_sexp;	/* solar radiation for oct */
+   SEXP plot_srad11_sexp;	/* solar radiation for nov */
+   SEXP plot_srad12_sexp;	/* solar radiation for dec */
+
+   /* added 25 new variables. */
 
    PROTECT( plot_sexp = AS_LIST( plot_sexp ) );
 
    plot_plot_sexp = get_list_element( plot_sexp, "plot" );
+
+   /* added latitude and longitude to the plots */
+   plot_lat_sexp  = get_list_element( plot_sexp, "lat" );
+   plot_lon_sexp  = get_list_element( plot_sexp, "lon" );
+
+
    plot_elev_sexp = get_list_element( plot_sexp, "elevation" );
    plot_slp_sexp  = get_list_element( plot_sexp, "slope" );
    plot_asp_sexp  = get_list_element( plot_sexp, "aspect" );
    plot_h20_sexp  = get_list_element( plot_sexp, "whc" );
    plot_map_sexp  = get_list_element( plot_sexp, "map" );
+   
    plot_si30_sexp = get_list_element( plot_sexp, "si30"); //site index
+   
+   /* todo: need to update the variables */
+   plot_gsp_sexp  = get_list_element( plot_sexp, "gsp" );
+   
+   plot_mt1_sexp  = get_list_element( plot_sexp, "mt1" );
+   plot_mt2_sexp  = get_list_element( plot_sexp, "mt2" );
+   plot_mt3_sexp  = get_list_element( plot_sexp, "mt3" );
+   plot_mt4_sexp  = get_list_element( plot_sexp, "mt4" );
+   plot_mt5_sexp  = get_list_element( plot_sexp, "mt5" );
+   plot_mt6_sexp  = get_list_element( plot_sexp, "mt6" );
+   plot_mt7_sexp  = get_list_element( plot_sexp, "mt7" );
+   plot_mt8_sexp  = get_list_element( plot_sexp, "mt8" );
+   plot_mt9_sexp  = get_list_element( plot_sexp, "mt9" );
+   plot_mt10_sexp  = get_list_element( plot_sexp, "mt10" );
+   plot_mt11_sexp  = get_list_element( plot_sexp, "mt11" );
+   plot_mt12_sexp  = get_list_element( plot_sexp, "mt12" );
+
+   plot_srad1_sexp  = get_list_element( plot_sexp, "srad1" );
+   plot_srad2_sexp  = get_list_element( plot_sexp, "srad2" );
+   plot_srad3_sexp  = get_list_element( plot_sexp, "srad3" );
+   plot_srad4_sexp  = get_list_element( plot_sexp, "srad4" );
+   plot_srad5_sexp  = get_list_element( plot_sexp, "srad5" );
+   plot_srad6_sexp  = get_list_element( plot_sexp, "srad6" );
+   plot_srad7_sexp  = get_list_element( plot_sexp, "srad7" );
+   plot_srad8_sexp  = get_list_element( plot_sexp, "srad8" );
+   plot_srad9_sexp  = get_list_element( plot_sexp, "srad9" );
+   plot_srad10_sexp  = get_list_element( plot_sexp, "srad10" );
+   plot_srad11_sexp  = get_list_element( plot_sexp, "srad11" );
+   plot_srad12_sexp  = get_list_element( plot_sexp, "srad12" );
+   
 
    PROTECT( plot_plot_sexp = coerceVector( plot_plot_sexp, INTSXP ) );
+
+   /* added for swo-hybrid */
+   PROTECT( plot_lat_sexp = coerceVector( plot_lat_sexp, REALSXP ) );
+   PROTECT( plot_lon_sexp = coerceVector( plot_lon_sexp, REALSXP ) );
+
    PROTECT( plot_elev_sexp = coerceVector( plot_elev_sexp, REALSXP ) );
+
    PROTECT( plot_slp_sexp  = coerceVector( plot_slp_sexp, REALSXP ) );
    PROTECT( plot_asp_sexp  = coerceVector( plot_asp_sexp, REALSXP ) );
    PROTECT( plot_h20_sexp  = coerceVector( plot_h20_sexp, REALSXP ) );
    PROTECT( plot_map_sexp  = coerceVector( plot_map_sexp, REALSXP ) );
    PROTECT( plot_si30_sexp = coerceVector( plot_si30_sexp, REALSXP ) ); //site index
 
+   /* protect the new variables for the SWOHybrid model */
+   PROTECT( plot_gsp_sexp = coerceVector( plot_gsp_sexp, REALSXP ) ); // growing season precip
+
+   PROTECT( plot_mt1_sexp = coerceVector( plot_mt1_sexp, REALSXP ) ); // mean monthly temp for jan
+   PROTECT( plot_mt2_sexp = coerceVector( plot_mt2_sexp, REALSXP ) ); // mean monthly temp for jan
+   PROTECT( plot_mt3_sexp = coerceVector( plot_mt3_sexp, REALSXP ) ); // mean monthly temp for jan
+   PROTECT( plot_mt4_sexp = coerceVector( plot_mt4_sexp, REALSXP ) ); // mean monthly temp for jan
+   PROTECT( plot_mt5_sexp = coerceVector( plot_mt5_sexp, REALSXP ) ); // mean monthly temp for jan
+   PROTECT( plot_mt6_sexp = coerceVector( plot_mt6_sexp, REALSXP ) ); // mean monthly temp for jan
+   PROTECT( plot_mt7_sexp = coerceVector( plot_mt7_sexp, REALSXP ) ); // mean monthly temp for jan
+   PROTECT( plot_mt8_sexp = coerceVector( plot_mt8_sexp, REALSXP ) ); // mean monthly temp for jan
+   PROTECT( plot_mt9_sexp = coerceVector( plot_mt9_sexp, REALSXP ) ); // mean monthly temp for jan
+   PROTECT( plot_mt10_sexp = coerceVector( plot_mt10_sexp, REALSXP ) ); // mean monthly temp for jan
+   PROTECT( plot_mt11_sexp = coerceVector( plot_mt11_sexp, REALSXP ) ); // mean monthly temp for jan
+   PROTECT( plot_mt12_sexp = coerceVector( plot_mt12_sexp, REALSXP ) ); // mean monthly temp for jan
+
+   PROTECT( plot_srad1_sexp = coerceVector( plot_srad1_sexp, REALSXP ) ); // solar radiation for jan
+   PROTECT( plot_srad2_sexp = coerceVector( plot_srad2_sexp, REALSXP ) ); // solar radiation for 
+   PROTECT( plot_srad3_sexp = coerceVector( plot_srad3_sexp, REALSXP ) ); // solar radiation for 
+   PROTECT( plot_srad4_sexp = coerceVector( plot_srad4_sexp, REALSXP ) ); // solar radiation for 
+   PROTECT( plot_srad5_sexp = coerceVector( plot_srad5_sexp, REALSXP ) ); // solar radiation for 
+   PROTECT( plot_srad6_sexp = coerceVector( plot_srad6_sexp, REALSXP ) ); // solar radiation for 
+   PROTECT( plot_srad7_sexp = coerceVector( plot_srad7_sexp, REALSXP ) ); // solar radiation for 
+   PROTECT( plot_srad8_sexp = coerceVector( plot_srad8_sexp, REALSXP ) ); // solar radiation for 
+   PROTECT( plot_srad9_sexp = coerceVector( plot_srad9_sexp, REALSXP ) ); // solar radiation for 
+   PROTECT( plot_srad10_sexp = coerceVector( plot_srad10_sexp, REALSXP ) ); // solar radiation for 
+   PROTECT( plot_srad11_sexp = coerceVector( plot_srad11_sexp, REALSXP ) ); // solar radiation for 
+   PROTECT( plot_srad12_sexp = coerceVector( plot_srad12_sexp, REALSXP ) ); // solar radiation for 
+
+   
+
+
    /* build the plots vector */
    *n_plots = length( plot_plot_sexp );
    plots_ptr = (struct PLOT_RECORD*)calloc( 
@@ -646,6 +894,11 @@ struct PLOT_RECORD *build_plot_array_from_sexp( SEXP plot_sexp,
    for( i = 0; i < (*n_plots); i++ )
    {
       plots_ptr[i].plot = INTEGER( plot_plot_sexp )[i];
+
+      /* added for the swo-hybrid model */
+      plots_ptr[i].latitude = REAL( plot_lat_sexp )[i];
+      plots_ptr[i].longitude = REAL( plot_lon_sexp )[i];
+
       plots_ptr[i].elevation = REAL( plot_elev_sexp )[i];
       plots_ptr[i].slope = REAL( plot_slp_sexp )[i];
       plots_ptr[i].aspect = REAL( plot_asp_sexp )[i];
@@ -670,6 +923,36 @@ struct PLOT_RECORD *build_plot_array_from_sexp( SEXP plot_sexp,
 	 }	 
       }
 
+      /* assign the remainder of the variables */
+      plots_ptr[i].growing_season_precip = REAL( plot_gsp_sexp )[i];  // assign the growing season precip */
+
+      /* assign the monthly temps and solar radiations */
+      plots_ptr[i].mean_monthly_temp[0] = REAL( plot_mt1_sexp )[i];
+      plots_ptr[i].mean_monthly_temp[1] = REAL( plot_mt2_sexp )[i];
+      plots_ptr[i].mean_monthly_temp[2] = REAL( plot_mt3_sexp )[i];
+      plots_ptr[i].mean_monthly_temp[3] = REAL( plot_mt4_sexp )[i];
+      plots_ptr[i].mean_monthly_temp[4] = REAL( plot_mt5_sexp )[i];
+      plots_ptr[i].mean_monthly_temp[5] = REAL( plot_mt6_sexp )[i];
+      plots_ptr[i].mean_monthly_temp[6] = REAL( plot_mt7_sexp )[i];
+      plots_ptr[i].mean_monthly_temp[7] = REAL( plot_mt8_sexp )[i];
+      plots_ptr[i].mean_monthly_temp[8] = REAL( plot_mt9_sexp )[i];
+      plots_ptr[i].mean_monthly_temp[9] = REAL( plot_mt10_sexp )[i];
+      plots_ptr[i].mean_monthly_temp[10] = REAL( plot_mt11_sexp )[i];
+      plots_ptr[i].mean_monthly_temp[11] = REAL( plot_mt12_sexp )[i];
+
+      plots_ptr[i].solar_radiation[0] = REAL( plot_srad1_sexp )[i];
+      plots_ptr[i].solar_radiation[1] = REAL( plot_srad2_sexp )[i];
+      plots_ptr[i].solar_radiation[2] = REAL( plot_srad3_sexp )[i];
+      plots_ptr[i].solar_radiation[3] = REAL( plot_srad4_sexp )[i];
+      plots_ptr[i].solar_radiation[4] = REAL( plot_srad5_sexp )[i];
+      plots_ptr[i].solar_radiation[5] = REAL( plot_srad6_sexp )[i];
+      plots_ptr[i].solar_radiation[6] = REAL( plot_srad7_sexp )[i];
+      plots_ptr[i].solar_radiation[7] = REAL( plot_srad8_sexp )[i];
+      plots_ptr[i].solar_radiation[8] = REAL( plot_srad9_sexp )[i];
+      plots_ptr[i].solar_radiation[9] = REAL( plot_srad10_sexp )[i];
+      plots_ptr[i].solar_radiation[10] = REAL( plot_srad11_sexp )[i];
+      plots_ptr[i].solar_radiation[11] = REAL( plot_srad12_sexp )[i];
+      
 /*       Rprintf( */
 /* 	 "%ld %lf %lf %lf %lf %lf\n", */
 /* 	 plots_ptr[i].plot, */
@@ -679,10 +962,11 @@ struct PLOT_RECORD *build_plot_array_from_sexp( SEXP plot_sexp,
 /* 	 plots_ptr[i].water_capacity, */
 /* 	 plots_ptr[i].mean_annual_precip ); */
 
+
    }
 
    /* this needs to match the number of PROTECT statements in the function */
-   UNPROTECT( 8 );
+   UNPROTECT( 8 + 25 + 2 );
    
    return plots_ptr;
 }
@@ -696,6 +980,10 @@ SEXP build_sexp_from_plot_array( unsigned long n_plots,
 
    /* plot variables */
    SEXP ret_plots_id;
+
+   SEXP ret_plots_lat;
+   SEXP ret_plots_lon;
+
    SEXP ret_plots_elev;
    SEXP ret_plots_slope;
    SEXP ret_plots_asp;
@@ -703,10 +991,42 @@ SEXP build_sexp_from_plot_array( unsigned long n_plots,
    SEXP ret_plots_map;
    SEXP ret_plots_si30;  //site index
 
-   PROTECT( ret_val = allocVector( VECSXP, 7 ) );
+   SEXP ret_plots_gsp;  // growing season precip
+
+   SEXP ret_plots_mt1;  // monthly temps
+   SEXP ret_plots_mt2;  // monthly temps
+   SEXP ret_plots_mt3;  // monthly temps
+   SEXP ret_plots_mt4;  // monthly temps
+   SEXP ret_plots_mt5;  // monthly temps
+   SEXP ret_plots_mt6;  // monthly temps
+   SEXP ret_plots_mt7;  // monthly temps
+   SEXP ret_plots_mt8;  // monthly temps
+   SEXP ret_plots_mt9;  // monthly temps
+   SEXP ret_plots_mt10;  // monthly temps
+   SEXP ret_plots_mt11;  // monthly temps
+   SEXP ret_plots_mt12;  // monthly temps
+
+   SEXP ret_plots_srad1;  // solar radation
+   SEXP ret_plots_srad2;  // solar radation
+   SEXP ret_plots_srad3;  // solar radation
+   SEXP ret_plots_srad4;  // solar radation
+   SEXP ret_plots_srad5;  // solar radation
+   SEXP ret_plots_srad6;  // solar radation
+   SEXP ret_plots_srad7;  // solar radation
+   SEXP ret_plots_srad8;  // solar radation
+   SEXP ret_plots_srad9;  // solar radation
+   SEXP ret_plots_srad10;  // solar radation
+   SEXP ret_plots_srad11;  // solar radation
+   SEXP ret_plots_srad12;  // solar radation
+
+   PROTECT( ret_val = allocVector( VECSXP, 7 + 25 + 2 ) );
 
    /* plots */
    PROTECT( ret_plots_id    = allocVector( INTSXP, n_plots ) );
+
+   PROTECT( ret_plots_lat  = allocVector( REALSXP, n_plots ) );
+   PROTECT( ret_plots_lon  = allocVector( REALSXP, n_plots ) );
+
    PROTECT( ret_plots_elev  = allocVector( REALSXP, n_plots ) );
    PROTECT( ret_plots_slope = allocVector( REALSXP, n_plots ) );
    PROTECT( ret_plots_asp   = allocVector( REALSXP, n_plots ) );
@@ -714,27 +1034,126 @@ SEXP build_sexp_from_plot_array( unsigned long n_plots,
    PROTECT( ret_plots_map   = allocVector( REALSXP, n_plots ) );
    PROTECT( ret_plots_si30  = allocVector( REALSXP, n_plots ) );  //site index
 
+   /* add the code for the new variables */
+   PROTECT( ret_plots_gsp  = allocVector( REALSXP, n_plots ) );  // growing season precip
+
+   /* monthly temps and solar radation */
+   PROTECT( ret_plots_mt1  = allocVector( REALSXP, n_plots ) );  // monthly temps
+   PROTECT( ret_plots_mt2  = allocVector( REALSXP, n_plots ) );  // monthly temps
+   PROTECT( ret_plots_mt3  = allocVector( REALSXP, n_plots ) );  // monthly temps
+   PROTECT( ret_plots_mt4  = allocVector( REALSXP, n_plots ) );  // monthly temps
+   PROTECT( ret_plots_mt5  = allocVector( REALSXP, n_plots ) );  // monthly temps
+   PROTECT( ret_plots_mt6  = allocVector( REALSXP, n_plots ) );  // monthly temps
+   PROTECT( ret_plots_mt7  = allocVector( REALSXP, n_plots ) );  // monthly temps
+   PROTECT( ret_plots_mt8  = allocVector( REALSXP, n_plots ) );  // monthly temps
+   PROTECT( ret_plots_mt9  = allocVector( REALSXP, n_plots ) );  // monthly temps
+   PROTECT( ret_plots_mt10  = allocVector( REALSXP, n_plots ) );  // monthly temps
+   PROTECT( ret_plots_mt11  = allocVector( REALSXP, n_plots ) );  // monthly temps
+   PROTECT( ret_plots_mt12  = allocVector( REALSXP, n_plots ) );  // monthly temps
+
+   PROTECT( ret_plots_srad1  = allocVector( REALSXP, n_plots ) );  // monthly solar radiation
+   PROTECT( ret_plots_srad2  = allocVector( REALSXP, n_plots ) );  // monthly solar radiation
+   PROTECT( ret_plots_srad3  = allocVector( REALSXP, n_plots ) );  // monthly solar radiation
+   PROTECT( ret_plots_srad4  = allocVector( REALSXP, n_plots ) );  // monthly solar radiation
+   PROTECT( ret_plots_srad5  = allocVector( REALSXP, n_plots ) );  // monthly solar radiation
+   PROTECT( ret_plots_srad6  = allocVector( REALSXP, n_plots ) );  // monthly solar radiation
+   PROTECT( ret_plots_srad7  = allocVector( REALSXP, n_plots ) );  // monthly solar radiation
+   PROTECT( ret_plots_srad8  = allocVector( REALSXP, n_plots ) );  // monthly solar radiation
+   PROTECT( ret_plots_srad9  = allocVector( REALSXP, n_plots ) );  // monthly solar radiation
+   PROTECT( ret_plots_srad10  = allocVector( REALSXP, n_plots ) );  // monthly solar radiation
+   PROTECT( ret_plots_srad11  = allocVector( REALSXP, n_plots ) );  // monthly solar radiation
+   PROTECT( ret_plots_srad12  = allocVector( REALSXP, n_plots ) );  // monthly solar radiation
+
    for( i = 0; i < n_plots; i++ )
    {
       INTEGER(ret_plots_id)[i] = plots_ptr[i].plot;
+
+      REAL(ret_plots_lat)[i] = plots_ptr[i].latitude;
+      REAL(ret_plots_lon)[i] = plots_ptr[i].longitude;
+
       REAL(ret_plots_elev)[i] = plots_ptr[i].elevation;
       REAL(ret_plots_slope)[i] = plots_ptr[i].slope;
       REAL(ret_plots_asp)[i] = plots_ptr[i].aspect;
       REAL(ret_plots_h20)[i] = plots_ptr[i].water_capacity;
       REAL(ret_plots_map)[i] = plots_ptr[i].mean_annual_precip;
       REAL(ret_plots_si30)[i] = plots_ptr[i].site_30; //site index
+
+      /* add te new variables */
+      REAL(ret_plots_gsp)[i] = plots_ptr[i].growing_season_precip; // growing season precip 
+
+      /* now, do the same for the monthly mean temp and solar radiation */
+      REAL(ret_plots_mt1)[i] = plots_ptr[i].mean_monthly_temp[0]; // monthly temp
+      REAL(ret_plots_mt2)[i] = plots_ptr[i].mean_monthly_temp[1]; // monthly temp
+      REAL(ret_plots_mt3)[i] = plots_ptr[i].mean_monthly_temp[2]; // monthly temp
+      REAL(ret_plots_mt4)[i] = plots_ptr[i].mean_monthly_temp[3]; // monthly temp
+      REAL(ret_plots_mt5)[i] = plots_ptr[i].mean_monthly_temp[4]; // monthly temp
+      REAL(ret_plots_mt6)[i] = plots_ptr[i].mean_monthly_temp[5]; // monthly temp
+      REAL(ret_plots_mt7)[i] = plots_ptr[i].mean_monthly_temp[6]; // monthly temp
+      REAL(ret_plots_mt8)[i] = plots_ptr[i].mean_monthly_temp[7]; // monthly temp
+      REAL(ret_plots_mt9)[i] = plots_ptr[i].mean_monthly_temp[8]; // monthly temp
+      REAL(ret_plots_mt10)[i] = plots_ptr[i].mean_monthly_temp[9]; // monthly temp
+      REAL(ret_plots_mt11)[i] = plots_ptr[i].mean_monthly_temp[10]; // monthly temp
+      REAL(ret_plots_mt12)[i] = plots_ptr[i].mean_monthly_temp[11]; // monthly temp
+
+      /* and the same for th esolar radidation */
+      REAL(ret_plots_srad1)[i] = plots_ptr[i].solar_radiation[0]; // solar radiation
+      REAL(ret_plots_srad2)[i] = plots_ptr[i].solar_radiation[1]; // solar radiation
+      REAL(ret_plots_srad3)[i] = plots_ptr[i].solar_radiation[2]; // solar radiation
+      REAL(ret_plots_srad4)[i] = plots_ptr[i].solar_radiation[3]; // solar radiation
+      REAL(ret_plots_srad5)[i] = plots_ptr[i].solar_radiation[4]; // solar radiation
+      REAL(ret_plots_srad6)[i] = plots_ptr[i].solar_radiation[5]; // solar radiation
+      REAL(ret_plots_srad7)[i] = plots_ptr[i].solar_radiation[6]; // solar radiation
+      REAL(ret_plots_srad8)[i] = plots_ptr[i].solar_radiation[7]; // solar radiation
+      REAL(ret_plots_srad9)[i] = plots_ptr[i].solar_radiation[8]; // solar radiation
+      REAL(ret_plots_srad10)[i] = plots_ptr[i].solar_radiation[9]; // solar radiation
+      REAL(ret_plots_srad11)[i] = plots_ptr[i].solar_radiation[10]; // solar radiation
+      REAL(ret_plots_srad12)[i] = plots_ptr[i].solar_radiation[11]; // solar radiation
+
    }
 
    SET_VECTOR_ELT( ret_val, 0, ret_plots_id );
-   SET_VECTOR_ELT( ret_val, 1, ret_plots_elev );
-   SET_VECTOR_ELT( ret_val, 2, ret_plots_slope );
-   SET_VECTOR_ELT( ret_val, 3, ret_plots_asp );
-   SET_VECTOR_ELT( ret_val, 4, ret_plots_h20 );
-   SET_VECTOR_ELT( ret_val, 5, ret_plots_map );
-   SET_VECTOR_ELT( ret_val, 6, ret_plots_si30  );
+
+   SET_VECTOR_ELT( ret_val, 1, ret_plots_lat );
+   SET_VECTOR_ELT( ret_val, 2, ret_plots_lon );
+
+   SET_VECTOR_ELT( ret_val, 3, ret_plots_elev );
+   SET_VECTOR_ELT( ret_val, 4, ret_plots_slope );
+   SET_VECTOR_ELT( ret_val, 5, ret_plots_asp );
+   SET_VECTOR_ELT( ret_val, 6, ret_plots_h20 );
+   SET_VECTOR_ELT( ret_val, 7, ret_plots_map );
+   SET_VECTOR_ELT( ret_val, 8, ret_plots_si30  );
+
+   /* you also have to update these for the outbound plots */
+   SET_VECTOR_ELT( ret_val, 9, ret_plots_gsp  );
+
+   SET_VECTOR_ELT( ret_val, 10, ret_plots_mt1  );
+   SET_VECTOR_ELT( ret_val, 11, ret_plots_mt2  );
+   SET_VECTOR_ELT( ret_val, 12, ret_plots_mt3  );
+   SET_VECTOR_ELT( ret_val, 13, ret_plots_mt4  );
+   SET_VECTOR_ELT( ret_val, 14, ret_plots_mt5  );
+   SET_VECTOR_ELT( ret_val, 15, ret_plots_mt6  );
+   SET_VECTOR_ELT( ret_val, 16, ret_plots_mt7  );
+   SET_VECTOR_ELT( ret_val, 17, ret_plots_mt8  );
+   SET_VECTOR_ELT( ret_val, 18, ret_plots_mt9  );
+   SET_VECTOR_ELT( ret_val, 19, ret_plots_mt10  );
+   SET_VECTOR_ELT( ret_val, 20, ret_plots_mt11  );
+   SET_VECTOR_ELT( ret_val, 21, ret_plots_mt12  );
+
+   SET_VECTOR_ELT( ret_val, 22, ret_plots_srad1  );
+   SET_VECTOR_ELT( ret_val, 23, ret_plots_srad2  );
+   SET_VECTOR_ELT( ret_val, 24, ret_plots_srad3  );
+   SET_VECTOR_ELT( ret_val, 25, ret_plots_srad4  );
+   SET_VECTOR_ELT( ret_val, 26, ret_plots_srad5  );
+   SET_VECTOR_ELT( ret_val, 27, ret_plots_srad6  );
+   SET_VECTOR_ELT( ret_val, 28, ret_plots_srad7  );
+   SET_VECTOR_ELT( ret_val, 29, ret_plots_srad8  );
+   SET_VECTOR_ELT( ret_val, 30, ret_plots_srad9  );
+   SET_VECTOR_ELT( ret_val, 31, ret_plots_srad10  );
+   SET_VECTOR_ELT( ret_val, 32, ret_plots_srad11  );
+   SET_VECTOR_ELT( ret_val, 33, ret_plots_srad12  );
 
    /* unprotect the vectros from the plots */
-   UNPROTECT( 7 );
+   UNPROTECT( 7 + 25 + 2 );
 
    return ret_val;
 }
@@ -996,6 +1415,7 @@ SEXP r_thin_sample(
 
    double x0;
    unsigned long age;
+   unsigned long n_years_projected;
    unsigned long n_plants;
    struct PLANT_RECORD *plants_ptr;
 
@@ -1023,6 +1443,8 @@ SEXP r_thin_sample(
 
    x0 = asReal( get_list_element( data_sexp, "x0" ) );
    age = asInteger( get_list_element( data_sexp, "age" ) );
+   n_years_projected = asInteger( get_list_element( data_sexp, "n.years.projected" ) );
+
    plots_ptr = build_plot_array_from_sexp( 
       get_list_element( data_sexp, "plots" ), &n_plots );
 
@@ -1060,6 +1482,7 @@ SEXP r_thin_sample(
 	    Rprintf( "Make sure you have the entry in your species map. See help\n" );
 	    ret_val = build_return_data_sexp( x0,
 					      age,
+                          n_years_projected,
 					      n_plots, plots_ptr, 
 					      n_plants, plants_ptr  );  
 	    
@@ -1141,6 +1564,7 @@ SEXP r_thin_sample(
 /*   Rprintf( "building return data sexp..." ); */
   ret_val = build_return_data_sexp( x0, 
 				    age,
+                    n_years_projected,
 				    n_plots, plots_ptr, 
 				    n_plants, plants_ptr  );  
 /*   Rprintf( "done\n" ); */
@@ -1153,7 +1577,7 @@ SEXP r_thin_sample(
 
 }
 
-SEXP r_fill_in_missing_values( 
+SEXP r_impute_missing_values( 
    SEXP data_sexp,
    SEXP ctl_sexp ) 
 {
@@ -1163,7 +1587,9 @@ SEXP r_fill_in_missing_values(
 
 /* data variables */
    double x0;
-   unsigned long age;
+   unsigned long age                = 0;
+   unsigned long n_years_projected  = 0;
+
    unsigned long n_plots;
    struct PLOT_RECORD *plots_ptr;
    unsigned long n_plants;
@@ -1182,11 +1608,10 @@ SEXP r_fill_in_missing_values(
    min_dbh  = asReal( get_list_element( ctl_sexp, "min.dbh" ) ); 
    baf  = asReal( get_list_element( ctl_sexp, "baf" ) ); 
 
-//   model_variant = asInteger( get_list_element( ctl_sexp, "model.variant" ) ); 
-
 /* set the data variables */
    x0 = asReal( get_list_element( data_sexp, "x0" ) );
    age = asInteger( get_list_element( data_sexp, "age" ) );
+   n_years_projected = asInteger( get_list_element( data_sexp, "n.years.projected" ) );
 
    plots_ptr = build_plot_array_from_sexp( 
       get_list_element( data_sexp, "plots" ), &n_plots );
@@ -1212,8 +1637,8 @@ SEXP r_fill_in_missing_values(
 
 
    /* if you have everything, then fill in the missing values */
-   fill_in_missing_values( &return_code,
-			   
+   //fill_in_missing_values( &return_code,
+    impute_missing_values( &return_code,
 			   N_SPECIES,
 			   SPECIES_PTR,
 			   N_COEFFS,
@@ -1238,6 +1663,7 @@ SEXP r_fill_in_missing_values(
 
   ret_val = build_return_data_sexp( x0, 
 				    age,
+                    n_years_projected,
 				    n_plots, plots_ptr, 
 				    n_plants, plants_ptr  );  
 /*   Rprintf( "done\n" ); */
@@ -1300,3 +1726,4 @@ SEXP r_calc_max_sdi(
 
 }
 
+// you might want to put the metric conversion function in the C code and put a wrapper here.
